<?php
/**
* Module Helper
*
* @author FlorianV
* @copyright (C) 2012 - Florian Voutzinos
* @email: voutzinos.florian|at|gmail.com
*
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
*
*/

defined('_JEXEC') or die('Restricted access');

// Loading models & tables
if (!class_exists( 'VirtueMartModelUser' )) require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_virtuemart'.DS.'models'.DS.'user.php');
if (!class_exists( 'VirtueMartModelOrders' )) require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_virtuemart'.DS.'models'.DS.'orders.php');
if (!class_exists('TableVmusers')) require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_virtuemart'.DS.'tables'.DS.'vmusers.php');
if (!class_exists('TableOrders')) require(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_virtuemart'.DS.'tables'.DS.'orders.php');

//jimport( 'joomla.utilities.date' );

/**
 * Handy object to display data in the view
 */
class ShippedOrder
{
	public $number;
	public $name;
	public $receiptDate;
	public $url;
	
	public function __construct($id, $number, $name, $receiptDate)
	{
		$this->number = $number;
		$this->name = $name;
		$this->receiptDate = $receiptDate;
		$this->url =& JRoute::_('index.php?option=com_virtuemart&view=orders&task=details&virtuemart_order_id='.$id);
	}
}

class ModInfoDeliveryHelper
{
	/**
	 * Detect problems regarding module configuration
	 */
	public function detectProblems($deliveryTime, $openedFrom, $openedTo, $monday, $tuesday, $wednesday, $thursday, $friday, $saturday, $sunday)
	{
		if($deliveryTime > 2000) // Do not use animals to deliver products http://www.wspa-international.org/
			return true;
		
		if($openedFrom > $openedTo)
			return true;
			
		if($monday == 0 && $tuesday == 0 && $wednesday == 0 && $thursday == 0 && $friday == 0 && $saturday == 0 && $sunday == 0 )
			return true;
		
		return false;
	}
	
	/**
	 * Select orders of the current user where status = shipped
	 * Return array of object matching virtuemart orders table or false if no shipped orders
	 */
	public function getUserShippedOrders()
	{
		$userModel = new VirtueMartModelUser();
		$uid = $userModel->getId();
		$ordersModel = new VirtueMartModelOrders();
	 	$orders = $ordersModel->getOrdersList($uid);
	
		$shippedOrdersCount = 0;

		foreach ($orders as $order)
		{
			if($order->order_status == 'S')
			{
				$shippedOrders[$shippedOrdersCount] = $order;
				$shippedOrdersCount++;
			}
		}
		
		if ($shippedOrdersCount == 0)
			return false;
		
		else
			return $shippedOrders;
	}
	
	/**
	 * Convert a float to a time in base 60
	 * Return array(0) = hours, array(1) = mins
	 */
	public function decimalToTime($decimalTime)
	{
		$hours = floor($decimalTime);
		$mins = round(60*($decimalTime-$hours));
		
		$timeArray[0] = (int)$hours;
		$timeArray[1] = (int)$mins;
		
		return $timeArray;
	}
	
	/**
	 * Return an array to easily use with date('N')
	 * (bool) $monday = 1 opened, 0 closed
	 */
	public function createWeekArray($monday, $tuesday, $wednesday, $thursday, $friday, $saturday, $sunday)
	{
		$weekArray = array();
		$weekArray[1] = $monday;
		$weekArray[2] = $tuesday;
		$weekArray[3] = $wednesday;
		$weekArray[4] = $thursday;
		$weekArray[5] = $friday;
		$weekArray[6] = $saturday;
		$weekArray[7] = $sunday;
		
		return $weekArray;
	}
	
	/**
	 * Call ShippedOrder constructor
	 * 
	 */
	public function createShippedOrderObject($order_id, $order_number, $order_name, $receiptDate)
	{
		return new ShippedOrder($order_id, $order_number, $order_name, $receiptDate);
	}
	
	
	/**
	 * Calculate the receipt date and time
	 * Return JDate Object
	 */
	public function parseDate($shippedOrders, $deliveryTime, $openedFromArray, $openedToArray, $weekArray)
	{
		// Assuming date of shipping = date of last order modification
		// Because if the order status change, it won't be displayed
		$ShipDateTime =& JFactory::getDate($shippedOrders->modified_on);
		
		$shipDateTimestamp = $ShipDateTime->getTimestamp();
		
		// From 1 = Monday to 7 = Sunday
		$shipDayNumber = (int)$ShipDateTime->format('N');
		
		// Receipt date = Ship Date + delivery time
		$ReceiptDateTime = clone $ShipDateTime;
		$addDeliveryTime = (string)'+ ' . $deliveryTime . ' hours';
		$ReceiptDateTime->modify($addDeliveryTime); // PHP 5.2.0
		$receiptDateTimestamp = $ReceiptDateTime->getTimestamp();
		
		// $OpenedFromDateTime and $OpenedToDateTime are clones of $ReceiptDateTime
		// because just used for a timestamp difference (we just consider the time)
		$OpenedFromDateTime = clone $ReceiptDateTime;
		$OpenedFromDateTime->setTime($openedFromArray[0], $openedFromArray[1]); // PHP 5.2.0
		$openedFromDateTimestamp = $OpenedFromDateTime->getTimestamp();

		$OpenedToDateTime = clone $ReceiptDateTime;
		$OpenedToDateTime->setTime($openedToArray[0], $openedToArray[1]);
		$openedToDateTimestamp = $OpenedToDateTime->getTimestamp();
		
		// If it might be received before opening time, we report to the opening
		if($receiptDateTimestamp < $openedFromDateTimestamp)
			$ReceiptDateTime->setTime($openedFromArray[0],$openedFromArray[1]);
		
		// If it might be received after closing time, we report to the next day at opening
		elseif($receiptDateTimestamp > $openedToDateTimestamp)
		{
			$ReceiptDateTime->modify('+1 day');
			$ReceiptDateTime->setTime($openedFromArray[0],$openedFromArray[1]);
		}
		
		// If it might be received beetween opening hours
		// It's not realistic to estimate precisely the receipt date, so we round minuts, and hours
		// eg : Receipt Time : 8:12 => 8:30
		// eg : Receipt Time : 8:41 => 9:00
		else{
			
			if ( (int)$ReceiptDateTime->format('i') < 30 )
			{
				$hours = (int)$ReceiptDateTime->format('H');
				$mins = 30;
				$ReceiptDateTime->setTime($hours, $mins);
			}
			
			elseif ( (int)$ReceiptDateTime->format('i') > 30 )
			{
				$hours = (int)$ReceiptDateTime->format('H');
				$hours ++;
				$mins = 0;
				$ReceiptDateTime->setTime($hours, $mins);
			}
		}
		
		// Now we have the time so we focus on the date
		// Creating a TempReceiptDateTime to decrement of one day by one day of opening until we go back to the shipdate
		$TempReceiptDateTime = clone $ReceiptDateTime;
		$TempReceiptDateTime->setTime(0,0);
		$ShipDateTime->setTime(0,0);
			
		while( ($TempReceiptDateTime->getTimestamp()-$ShipDateTime->getTimestamp()) !=0 )
		{
			// If it's closed
			if(!$weekArray[$shipDayNumber])
				$ReceiptDateTime->modify('+1 day');
					
			else
				$TempReceiptDateTime->modify('-1 day');
				
			$shipDayNumber++;
			
			if($shipDayNumber == 8) 
				$shipDayNumber = 1;
		}
		
		// Now we check the real receipt date
		$receiptDayNumber = (int)$ReceiptDateTime->format('N');
		
		// If it's closed
		while( !$weekArray[$receiptDayNumber] )
		{
			$ReceiptDateTime->modify('+1 day');
			$receiptDayNumber++;
			
			if($receiptDayNumber == 8) 
				$receiptDayNumber = 1;
		}
		
		// If the order might be received before today, we don't print it
		$now =& JFactory::getDate();
		
		if ( $now->getTimestamp() > $ReceiptDateTime->getTimestamp() )
			return false;
	
		else
			return $ReceiptDateTime;
	}
	
	/**
	 * Format JDate Object according to dateFormat or advancedDateformat if specified and valid
	 * Return formated date string
	 */
	public function formatDate($JDate, $dateFormat, $advancedDateFormat)
	{
		// If advancedateFormat not specified or invalid, we consider the dateformat param
		if( ($advancedDateFormat == "") || (!$JDate->format($advancedDateFormat)) )
		{
		
			switch($dateFormat)
			{
				case 1:
				$dateFormat = 'd-m-Y H:i'; // Day-Month-Year Time
				break;
			
				case 2:
				$dateFormat = 'm-d-Y H:i'; // Month-Day-Year Time
				break;
			
				case 3:
				$dateFormat = 'D, j M H:i'; // Sun, 28 Jan 08:00
				break;
			
				case 4:
				$dateFormat = 'l, j F H:i'; // Sunday, 28 January 08:00
				break;
			
				case 5:
				$dateFormat = 'D, j M Y H:i'; // Sun, 28 Jan 2012 08:00
				break;
			
				case 6:
				$dateFormat = 'l, j F Y H:i'; // Sunday, 28 January 2012 08:00
				break;
			}

			return $JDate->format($dateFormat, $local = true, $translate = true);
		}
		
		else
			return $JDate->format($advancedDateFormat, $local = true, $translate = true);
	}
}